
# https://github.com/ResearchObject/bagit-ro
# This GitHub repository explains by example a profile for a BagIt bag to also be a Research Object.


# hydroshare uses bagit.

# if we need to 'package' data this may be a good form

